"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_express_1 = require("apollo-server-express");
const express_1 = __importDefault(require("express"));
const logger_1 = require("../logger");
const loaders_1 = require("../loaders");
const database_connector_1 = __importDefault(require("../connectors/database-connector"));
const rest_loader_1 = __importDefault(require("../loaders/rest.loader"));
const auth_directive_1 = __importDefault(require("./auth.directive"));
class App {
    constructor(config) {
        this.config = config;
        this.gqlLoader = new loaders_1.GqlLoader();
        this.expressApp = express_1.default();
    }
    async buildServer(initContextStorage) {
        try {
            const connection = await this.createDbConnectionAndMigrate();
            await this.buildGraphql(connection);
            const endpoints = await this.buildRest(connection);
            this.expressApp.use('/health', (req, res) => res.json());
            if (initContextStorage) {
                initContextStorage(this.expressApp);
            }
            this.expressApp.listen(this.config.port, () => {
                logger_1.logger.log(`GraphQL is available on: http://localhost:${this.config.port}/graphql`, 'Bootstrap');
                if (endpoints.length) {
                    logger_1.logger.log('REST endpoints are available on:', 'Bootstrap');
                    const endpointsFlatten = {};
                    endpoints.forEach((endpoint) => {
                        (endpointsFlatten[endpoint.route] = endpointsFlatten[endpoint.route] || []).push(endpoint.method);
                    });
                    Object.entries(endpointsFlatten).forEach((endpoint) => {
                        logger_1.logger.log(`  -> [${endpoint[1].join(', ').toUpperCase()}] http://localhost:${this.config.port}/rest${endpoint[0]}`, 'Bootstrap');
                    });
                }
            });
        }
        catch (e) {
            throw new Error(`Server failed to start, error: ${e}`);
        }
    }
    async buildGraphql(connection) {
        const contextHandler = await this.gqlLoader.getContextHandler();
        const server = await new apollo_server_express_1.ApolloServer({
            playground: {
                endpoint: this.config.playgroundUrl,
            },
            introspection: true,
            typeDefs: await this.gqlLoader.getTypeDefs(),
            resolvers: await this.gqlLoader.getResolvers(),
            uploads: false,
            schemaDirectives: {
                auth: auth_directive_1.default,
            },
            context: async (expressContext) => {
                const context = {
                    db: { connection },
                    request: expressContext.req,
                };
                if (contextHandler && expressContext.req.body.operationName !== 'IntrospectionQuery') {
                    const result = await contextHandler(context);
                    if (result && typeof result === 'object') {
                        return Object.assign(Object.assign({}, context), result);
                    }
                }
                return context;
            },
        });
        server.applyMiddleware({ app: this.expressApp });
    }
    async buildRest(connection) {
        return new rest_loader_1.default(this.expressApp, { db: { connection } }).loadEndpoints();
    }
    async createDbConnectionAndMigrate() {
        if (this.config.skipDatabase) {
            logger_1.logger.log('Skipping connection to database, to disable - remove skipDatabase flag from codestore.yaml', 'Database');
            return undefined;
        }
        logger_1.logger.log('Connecting to database', 'Database');
        const connection = await database_connector_1.default.createConnection(this.config.db);
        logger_1.logger.log('Successfully connected', 'Database');
        if (!this.config.db.synchronize) {
            await connection.runMigrations();
            logger_1.logger.log('Migrations ran', 'Database');
        }
        return connection;
    }
}
exports.default = App;
//# sourceMappingURL=app.js.map